import { useEffect, useState, memo } from "react";
import { makeStyles, ThemeProviderDefault } from "./theme";
import type { ReactNode } from "react";
import { useSplashScreen } from "onyxia-ui";
import type { ComponentType } from "./tools/ComponentType";
import type { ThemeProviderProps } from "onyxia-ui";
import { useIsThemeProvided } from "onyxia-ui/lib/ThemeProvider";
import { useDomRect } from "onyxia-ui";
import { useEvt } from "evt/hooks/useEvt";
import { Evt } from "evt";
import { changeColorOpacity } from "onyxia-ui";
import { GlLinkToTop } from "./utils/GlLinkToTop";
import { disableEmotionWarnings } from "./tools/disableEmotionWarnings";
import type { CSSObject } from "tss-react/types";
import { useGetScrollableParent } from "./tools/useGetScrollableParent";

disableEmotionWarnings();

export type HeaderOptions =
    | HeaderOptions.Fixed
    | HeaderOptions.TopOfPage
    | HeaderOptions.Sticky;

export namespace HeaderOptions {
    export type Fixed = {
        position: "fixed";
        isRetracted?: boolean | "smart";
    };

    export type TopOfPage = {
        position: "top of page";
        isRetracted?: boolean;
    };

    export type Sticky = {
        position: "sticky";
        isRetracted?: boolean | "smart";
    };
}

export type GlTemplateProps = {
    header?: ReactNode;
    footer?: ReactNode;
    SplashScreenLogo?: ComponentType<{ className: string }>;
    splashScreenLogoFillColor?: string;
    children: ReactNode;
    ThemeProvider?: ComponentType<{
        splashScreen?: NonNullable<ThemeProviderProps["splashScreen"]>;
        children: ReactNode;
    }>;
    headerOptions?: HeaderOptions;
    className?: string;
    hasTopOfPageLinkButton?: boolean;
    classes?: Partial<ReturnType<typeof useStyles>["classes"]>;
};

const GlTemplateInner = memo(
    (
        props: Omit<GlTemplateProps, "SplashScreenLogo"> & {
            isThemeProvidedOutside: boolean;
        },
    ) => {
        const {
            header,
            isThemeProvidedOutside,
            children,
            footer,
            className,
            hasTopOfPageLinkButton,
        } = props;

        const headerOptions: Required<HeaderOptions> = (() => {
            const { headerOptions } = props;

            if (headerOptions === undefined) {
                return {
                    "position": "top of page",
                    "isRetracted": false,
                } as const;
            }

            switch (headerOptions.position) {
                case "fixed":
                    return {
                        ...headerOptions,
                        "isRetracted": headerOptions.isRetracted ?? false,
                    };
                case "top of page":
                    return {
                        ...headerOptions,
                        "isRetracted": headerOptions.isRetracted ?? false,
                    };
                case "sticky":
                    return {
                        ...headerOptions,
                        "isRetracted": headerOptions.isRetracted ?? false,
                    };
            }
        })();

        {
            const { hideRootSplashScreen } = useSplashScreen();

            useEffect(() => {
                if (isThemeProvidedOutside) {
                    return;
                }

                hideRootSplashScreen();
            }, []);
        }

        const {
            ref: headerWrapperRef,
            domRect: { height: headerHeight },
        } = useDomRect();
        const {
            ref: childrenWrapperRef,
            domRect: { width: childrenWrapperWidth },
        } = useDomRect();

        const { ref: rootRef, scrollableParent } = useGetScrollableParent();

        const [isSmartHeaderVisible, setIsSmartHeaderVisible] = useState(true);

        useEvt(
            ctx => {
                if (scrollableParent === undefined) {
                    return;
                }
                let previousScrollTop = 0;

                Evt.from(ctx, window, "scroll").attach(() => {
                    const scrollTop = (() => {
                        if (scrollableParent === window) {
                            return scrollableParent.scrollY;
                        }
                        return (scrollableParent as HTMLElement).scrollTop;
                    })();

                    setIsSmartHeaderVisible(
                        scrollTop < previousScrollTop
                            ? true
                            : scrollTop <= headerHeight,
                    );

                    previousScrollTop = scrollTop;
                });
            },
            [headerHeight, headerOptions.isRetracted, scrollableParent],
        );

        const { classes, cx } = useStyles(
            {
                childrenWrapperWidth,
                headerHeight,
                "isHeaderRetracted":
                    headerOptions.isRetracted === "smart"
                        ? !isSmartHeaderVisible
                        : headerOptions.isRetracted,
                "headerPosition": headerOptions.position,
            },
            { props },
        );

        return (
            <div ref={rootRef} className={cx(classes.root, className)}>
                <div ref={headerWrapperRef} className={classes.headerWrapper}>
                    {header}
                </div>
                <div
                    className={classes.childrenWrapper}
                    ref={childrenWrapperRef}
                >
                    {children}
                    {hasTopOfPageLinkButton && <GlLinkToTop />}
                    <div className={classes.footerWrapper}>{footer}</div>
                </div>
            </div>
        );
    },
);

export const GlTemplate = memo((props: GlTemplateProps) => {
    const {
        ThemeProvider = ThemeProviderDefault,
        SplashScreenLogo,
        splashScreenLogoFillColor,
        ...rest
    } = props;

    const isThemeProvided = useIsThemeProvided();

    const children = (
        <GlTemplateInner {...rest} isThemeProvidedOutside={isThemeProvided} />
    );

    return isThemeProvided ? (
        children
    ) : (
        <ThemeProvider
            splashScreen={
                SplashScreenLogo === undefined
                    ? undefined
                    : {
                          "Logo": SplashScreenLogo,
                          "fillColor": splashScreenLogoFillColor,
                          "minimumDisplayDuration": 0,
                      }
            }
        >
            {children}
        </ThemeProvider>
    );
});

const useStyles = makeStyles<{
    headerHeight: number;
    childrenWrapperWidth: number;
    isHeaderRetracted: boolean;
    headerPosition: Required<HeaderOptions>["position"];
}>({ "name": { GlTemplate } })(
    (
        theme,
        {
            headerHeight,
            childrenWrapperWidth,
            isHeaderRetracted,
            headerPosition,
        },
    ) => {
        return {
            "root": {
                "overflowX": "hidden",
            },
            "headerWrapper": {
                "padding": theme.spacing({
                    "rightLeft": `${theme.paddingRightLeft}px`,
                    "topBottom": `${theme.spacing(3)}px`,
                }),
                ...(() => {
                    let out: CSSObject = {};
                    if (
                        headerPosition === "fixed" ||
                        headerPosition === "sticky"
                    ) {
                        out = {
                            ...out,
                            "width": childrenWrapperWidth,
                            "backgroundColor": changeColorOpacity({
                                "color":
                                    theme.colors.useCases.surfaces.background,
                                "opacity": 0.94,
                            }),
                            "top": !isHeaderRetracted ? 0 : -headerHeight,
                            "transition": "top 350ms",
                        };
                    }
                    switch (headerPosition) {
                        case "fixed":
                            out = {
                                ...out,
                                "position": "fixed",
                            };
                            break;
                        case "sticky":
                            out = {
                                ...out,
                                "position": "sticky",
                                "pointerEvents": isHeaderRetracted
                                    ? "none"
                                    : undefined,
                            };
                            break;
                        case "top of page":
                            return {};
                    }

                    return out;
                })(),
            },

            "footerWrapper": {
                "marginTop": "auto",
            },
            "childrenWrapper": {
                "display": "flex",
                "flexDirection": "column",
                "& > :first-child": {
                    "paddingTop":
                        headerPosition === "fixed" ? headerHeight : undefined,
                },
                "minHeight": window.innerHeight,
            },
        };
    },
);
